import pandas as pd
from typing import List, Dict, Union, Callable
from get_pop.definitions import PATH_USA_POP, DIR_DATA
import logging
import pathlib
from get_pop.definitions import selected_fields_type, selected_values_type


def parse_states(
    value_field: str,
    selected_values: selected_values_type,
    selected_fields: selected_fields_type,
    save_dir: Union[pathlib.Path, str],
    *,
    field_cleaners: Dict[str, Callable[[pd.DataFrame, str], pd.DataFrame]] = None,
    file_partial: str = None,
) -> None:
    """
    Outputs CSVs of state data after parsing a large CSV of U.S. county-level census data for selected states.

    Args:
        value_field (str): Field that will be used to filter data by.
        selected_values (selected_values_type): A list of dictionaries relating to the state's selected for data
            extraction. Each dict has a key-value pairs for the full name of the state and it's two-letter abbreviation.
        selected_fields (selected_fields_type): A list of dictionaries that represent the fields that will be selected from
            the U.S. Census CSV, and how the field will be represented in the final CSV.
        save_dir: Union[pathlib.Path, str]: Path where processed state CSVs will be saved
        field_cleaners (Dict[Callable[[pd.DataFrame, str], pd.DataFrame]]): (Optional) function that cleans a
            specified field
        file_partial (str): (Optional) String that is used to determine part of final CSV filename for each state.

    Returns:
        None. Output is saved as CSV files.
    """

    # read
    df = pd.read_csv(PATH_USA_POP, encoding="ISO-8859-1")

    # filter - remove statewide population counts
    df = df[df["COUNTY"] != 0]

    # filter - include only selected values
    selected_values_names = [x["name"] for x in selected_values]
    df = df[df[value_field].isin(selected_values_names)]

    # option - clean value field
    if field_cleaners:
        for field in field_cleaners.keys():
            cleaner_func = field_cleaners[field]
            df = cleaner_func(df, field)

    # rename field lookuptable
    rename_schema = {}
    for field in selected_fields:
        input_name = field["input_name"]
        output_name = field["output_name"]
        rename_schema[input_name] = output_name

    # group by
    by_state = df.groupby(value_field)

    for name, group in by_state:
        logging.info(f"Processing: {name}")

        # get selected state dict for processing instructions
        selected_state = list(filter(lambda x: x["name"] == name, selected_values))[0]

        # generate FIPS code
        # Temporarily disabling SettingWithCopy warning
        pd.reset_option("mode.chained_assignment")
        with pd.option_context("mode.chained_assignment", None):
            group["STATE"] = group["STATE"].astype(str).str.zfill(2)
            group["COUNTY"] = group["COUNTY"].astype(str).str.zfill(3)
            group["FIPS"] = group["STATE"] + group["COUNTY"]

        # truncate cols in df
        selected_fields_input = [x["input_name"] for x in selected_fields]
        group = group[selected_fields_input]

        # rename
        group = group.rename(columns=rename_schema)

        # option - special processor (special funcs for doing extra stuff to df)
        special_processors = selected_state.get("special_processors")
        if special_processors:
            for processor in special_processors:
                group = processor(group)

        # produce csv
        abbrv = selected_state["abbrv"]
        filename = (
            f"{abbrv}-{file_partial}-pop.csv" if file_partial else f"{abbrv}-pop.csv"
        )
        save_dir = pathlib.Path(save_dir) if isinstance(save_dir, str) else save_dir
        save_dir.mkdir(exist_ok=True)
        output_path = save_dir / filename
        group.to_csv(output_path, index=False)



import logging
from typing import List, Union

from get_pop.modules.helper.misc import check_and_clear_dir
from get_pop.static.constants import (
    state_index,
    value_field,
    field_cleaners,
    default_fields,
)
import pathlib
from get_pop.definitions import selected_fields_type
from get_pop.definitions import DIR_DATA
from get_pop.modules.init.init_program import init_program
from get_pop.modules.parse.parse import parse_states

clean_states = [x["abbrv"] for x in state_index]


def get_pop(
    states: List[str],
    *,
    save_dir: Union[pathlib.Path, str] = DIR_DATA,
    clear_dir: bool = False,
    selected_fields: selected_fields_type = default_fields,
) -> None:
    """
    Takes a list of 2-letter US state postal codes, returns CSVs for each state.

    Args:
        states List[str]: List of states.
        save_dir Union[pathlib.Path, str]: (Optional) Absolute path where CSV data will be stored. Defaults to /data
            in current working directory.
        clear_dir (bool):
        selected_fields (selected_fields_type): (Optional) A list of dictionaries with fields that should be
            included in the final CSV for each state. Default fields: "fips", "name", "population"

    Returns:
        None.
    """

    # init
    package_name = "getpop"
    init_program(package_name)

    # process states
    logging.info(f"Selected states: {states}")
    selected_states = []
    if "all" in states[0]:
        selected_states = state_index
    else:
        for state in states:
            index_item = [
                x for x in state_index if x["abbrv"].lower() in state.lower()
            ][0]
            selected_states.append(index_item)

    # clear directory flag
    if clear_dir:
        check_and_clear_dir(save_dir)

    parse_states(
        value_field=value_field,
        selected_values=selected_states,
        selected_fields=selected_fields,
        field_cleaners=field_cleaners,
        file_partial="county",
        save_dir=save_dir,
    )

    # end
    logging.info(f"{package_name} complete")



from get_pop.modules.parse.helpers import clean_county, merge_nyc_boroughs

value_field = "STNAME"
field_cleaners = {"CTYNAME": clean_county}
state_index = [
    {"name": "Alabama", "abbrv": "al"},
    {"name": "Alaska", "abbrv": "ak"},
    {"name": "Arizona", "abbrv": "az"},
    {"name": "Arkansas", "abbrv": "ar"},
    {"name": "California", "abbrv": "ca"},
    {"name": "Colorado", "abbrv": "co"},
    {"name": "Connecticut", "abbrv": "ct"},
    {"name": "Delaware", "abbrv": "de"},
    {"name": "Florida", "abbrv": "fl"},
    {"name": "Georgia", "abbrv": "ga"},
    {"name": "Hawaii", "abbrv": "hi"},
    {"name": "Idaho", "abbrv": "id"},
    {"name": "Indiana", "abbrv": "in"},
    {"name": "Iowa", "abbrv": "ia"},
    {"name": "Kansas", "abbrv": "ks"},
    {"name": "Kentucky", "abbrv": "ky"},
    {"name": "Louisiana", "abbrv": "la"},
    {"name": "Maine", "abbrv": "me"},
    {"name": "Maryland", "abbrv": "md"},
    {"name": "Massachusetts", "abbrv": "ma"},
    {"name": "Michigan", "abbrv": "mi"},
    {"name": "Minnesota", "abbrv": "mn"},
    {"name": "Mississippi", "abbrv": "ms"},
    {"name": "Missouri", "abbrv": "mo"},
    {"name": "Montana", "abbrv": "mt"},
    {"name": "Nebraska", "abbrv": "ne"},
    {"name": "Nevada", "abbrv": "nv"},
    {"name": "New Hampshire", "abbrv": "nh"},
    {"name": "New Jersey", "abbrv": "nj"},
    {"name": "New Mexico", "abbrv": "nm"},
    {"name": "New York", "abbrv": "ny", "special_processors": [merge_nyc_boroughs]},
    {"name": "New Jersey", "abbrv": "nj"},
    {"name": "North Carolina", "abbrv": "nc"},
    {"name": "North Dakota", "abbrv": "nd"},
    {"name": "Ohio", "abbrv": "oh"},
    {"name": "Oklahoma", "abbrv": "ok"},
    {"name": "Oregon", "abbrv": "or"},
    {"name": "Pennsylvania", "abbrv": "pa"},
    {"name": "Rhode Island", "abbrv": "ri"},
    {"name": "South Carolina", "abbrv": "sc"},
    {"name": "South Dakota", "abbrv": "sd"},
    {"name": "Tennessee", "abbrv": "tn"},
    {"name": "Texas", "abbrv": "tx"},
    {"name": "Utah", "abbrv": "ut"},
    {"name": "Vermont", "abbrv": "vt"},
    {"name": "Virginia", "abbrv": "va"},
    {"name": "Washington", "abbrv": "wa"},
    {"name": "West Virginia", "abbrv": "wv"},
    {"name": "Wisconsin", "abbrv": "wi"},
    {"name": "Wyoming", "abbrv": "wy"},
]

default_fields = [
    {"input_name": "FIPS", "output_name": "fips"},
    {"input_name": "CTYNAME", "output_name": "name"},
    {"input_name": "POPESTIMATE2019", "output_name": "population"},
]



from pathlib import Path
from shutil import copyfile
import logging


def move_files(dir_from: Path, dir_to: Path):
    logging.info(f"Moving files from '{dir_from}' to '{dir_to}'")
    p = dir_from.glob("**/*")
    input_paths = [x for x in p if x.is_file()]
    for input_path in input_paths:
        filename = input_path.name
        output_path = dir_to / filename
        copyfile(input_path, output_path)
        logging.info(f"Moved file: {filename}")



import pandas as pd


def clean_county(df: pd.DataFrame, field: str) -> pd.DataFrame:
    """
    Cleans up the text in specified field.

    Args:
        df (pd.Dataframe): Dataframe of data.
        field (str): Field to clean up.

    Returns:
        pd.Dataframe with cleaned field.
    """
    df[field] = df[field].str.replace(" County", "")
    return df


def merge_nyc_boroughs(df: pd.DataFrame) -> pd.DataFrame:
    """
    Combines population in NY state's five buroughs to create a new row representing the population of New York City.

    Args:
        df (pd.Dataframe): A dataframe with NY data

    Returns:
        pd.Dataframe with NYC population.
    """
    df_buroughs = df.loc[
        df["name"].isin(["Bronx", "Queens", "New York", "Kings", "Richmond"])
    ]
    nyc_pop = df_buroughs["population"].sum()
    return df.append(
        {"name": "New York City", "population": nyc_pop}, ignore_index=True
    )



import logging
from dotenv import load_dotenv
from get_pop.definitions import DIR_DATA
from get_pop.logs.config.logging import logs_config
from get_pop.modules.helper.misc import delete_dir_contents
from get_pop.modules.helper.time import utc_now
from get_pop.modules.init.pandas_opts import pandas_opts
from datetime import datetime


def init_program(package_name: str = None) -> datetime:
    """
    Initializes program, inits logging, performs filesystem clean up, etc.

    Args:
        package_name (str): (optional) name of package

    Returns:
        A datetime object with the time the program started.
    """

    # Load env vars
    load_dotenv()

    # init logging
    program_start_time = utc_now()
    timezone = program_start_time.tzinfo
    logs_config()
    if package_name:
        logging.info(f"Initializing {package_name}")

    pandas_opts()

    return program_start_time



import pandas as pd


def pandas_opts() -> None:
    """
    Sets desired pandas options.

    Returns:
        None
    """
    pd.set_option("display.max_columns", 20)
    pd.set_option("display.width", 2000)
    pd.set_option("display.max_rows", 2000)



import os
import shutil
from typing import Union
from pathlib import Path
import logging


def check_and_clear_dir(dir_path: Union[str, Path]) -> None:
    """
    Checks if directory exists, if it does then delete contents of directory but keeps directory.

    Args:
        dir_path (str, pathlib.Path): Absolute path of directory to target.

    Returns:
        None
    """
    # create or clean download dir
    if isinstance(dir_path, str):
        dir_path = Path(dir_path)
    if dir_path.is_dir():
        # delete files from previous run
        delete_dir_contents(dir_path)
    else:
        logging.info(f"Path {dir_path} is not a directory and can't be cleared")


def delete_dir_contents(dir_path: Union[str, Path]) -> None:
    """
    Deletes all files and folders within a directory.

    Args:
        dir_path (str, Path): Absolute path of directory to target.
    """
    if isinstance(dir_path, str):
        dir_path = Path(dir_path)

    for filename in os.listdir(dir_path):
        file_path = os.path.join(dir_path, filename)
        try:
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except Exception as e:
            print("Failed to delete %s. Reason: %s" % (file_path, e))



from pathlib import Path
import click
from typing import Union
from get_pop.definitions import DIR_DATA
from get_pop.get_pop import get_pop
from get_pop.static.constants import state_index

clean_states = [x["abbrv"] for x in state_index] + ["all"]


@click.command()
@click.argument("states", nargs=-1, type=click.Choice(clean_states))
@click.option(
    "--save-dir",
    "--dir",
    default=DIR_DATA,
    help="Absolute path of directory where CSV files will be output. Defaults to saving them in /data in the current "
    "working directory",
)
@click.option(
    "--clear-dir",
    "--cdir",
    is_flag=True,
    help="Deletes all existing files in save_dir path. Defaults to false.",
)
def main(states: tuple, save_dir: Union[str, Path], clear_dir: bool) -> None:
    """
    CLI API using click library. Generates CSVs of county-level state data based on
    provided arguments.

    Args:
        states (tuple): Tuple of one or more two-letter state abbreviations. Click takes a user's input
            from the command line and stores it as a tuple.
        save_dir (Union[str, Path]): (Optional) Path to directory where CSVs will be saved. Defaults to saving them in
            /data in the current working directory
        clear_dir (bool): (Optional) Deletes all existing files in save_dir path. Defaults to false

    Returns:
        None
    """
    states_list = list(states)  # convert from tuple to list
    get_pop(states_list, save_dir=save_dir, clear_dir=clear_dir)



from setuptools import setup, find_packages
from get_pop import __version__, __author__, __project__

requires = [
    "pyyaml",
    "python-dotenv",
    "pandas",
    "iso8601",
    "click",
    "mypy-extensions",
]

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name=__project__,
    version=__version__,
    author=__author__,
    description="A python command line utility that generates CSVs of county-level population data for specified US "
    "states.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
    ],
    keywords=["population csv geography usa counties"],
    # list folders, not files
    packages=find_packages(exclude=["docs", "tests*"]),
    install_requires=requires,
    entry_points={
        "console_scripts": [
            "getpop=get_pop.cli:main",
            "get_pop=get_pop.cli:main",
            "get-pop=get_pop.cli:main",
        ]
    },
    package_data={"get_pop": ["static/*", "logs/config/logging.yaml"]},
)



"""
This file contains important project variables.
"""
import os
from pathlib import Path
from typing import List, Dict, Union, Callable
from mypy_extensions import TypedDict

# This sets our root directory as the project directory
PACKAGE_DIR = Path(
    os.path.dirname(os.path.abspath(__file__))
)  # This is the main package root
CWD = Path(os.getcwd())

# DIRECTORIES
DIR_LOGS = PACKAGE_DIR / "logs"  # main dir for log-related files
DIR_LOGS_OUTPUT = DIR_LOGS / "output"
DIR_LOGS_CONFIG = DIR_LOGS / "config"
DIR_DATA = CWD / "data"
DIR_STATIC = PACKAGE_DIR / "static"

# PATHS
PATH_LOGS_CONFIG = DIR_LOGS_CONFIG / "logging.yaml"
PATH_LOGS_CONFIG_TEST = DIR_LOGS_CONFIG / "logging_test.yaml"
PATH_USA_POP = DIR_STATIC / "usa_pop_counties_2019.csv"

# TYPES
selected_values_type = List[TypedDict("state_dict", {"name": str, "abbrv": str})]
selected_fields_type = List[
    TypedDict("field_dict", {"input_name": str, "output_name": str})
]
